using UnityEngine;
using System.Collections;

using System.Collections.Generic;

public class Game : MonoBehaviour {	
	
	public float framelength=0.4f;
	public Triple.Direction playerforward = Triple.Direction.East;
	public Triple.Direction playerup = Triple.Direction.Up;
	public GameObject beamprefab;
	public GameObject player;
	public int width = 10;
	public Triple playerpos;
	public GameObject boundingbox;
	public Triple blippos;
	public int playerlength=0;
	
	public AudioClip sndtick;
	public AudioClip sndcollect;
	public AudioClip snddead;
	
	public AudioSource source;
	
	private List<GameObject> beamobs;
	private List<Triple.Direction> beamdirs;
	private List<Triple.Direction> beamdirs2;
	private List<Triple> beamposes;
	
	public GameObject blip;
	private GameObject beambox;
	
	public int inittraillength=0;
	public Vector3 PointToCoord(Triple t)
	{
		return new Vector3(t.x+0.5f,t.y+0.5f,t.z+0.5f);
	}
	
	public Vector3 PointToVector(Triple t)
	{
		return new Vector3(t.x,t.y,t.z);
	}
	private bool dead=false;
	private static bool awake=false;
	// Use this for initialization
	void Awake () {
		player = gameObject;
		playerpos = new Triple(width/2,width/2,width/2);
		blippos = new Triple(width/2,width/2+3,width/2);
		beambox = new GameObject("beambox");
		
		boundingbox.transform.localScale = Vector3.one*width;
		boundingbox.transform.position = boundingbox.transform.localScale/2;
		beamobs = new List<GameObject>();
		beamdirs = new List<Triple.Direction>();
		beamdirs2 = new List<Triple.Direction>();
		beamposes = new List<Triple>();
		playerlength=inittraillength;
		
		if (awake==false)
		{
			tm.gameObject.active=true;
			int highscore = PlayerPrefs.GetInt("highscore",0);
			tm.text="3NAKE\narrow keys to move\nr to restart\npress SPACE to begin\n\n highscore:"+highscore;
		}
		SetBlipPos();
	}
	
	
	void SetBlipPos()
	{
		blippos = new Triple(Random.Range(0,width),Random.Range(0,width),Random.Range(0,width));
		blip.transform.position = PointToCoord(blippos);
		TryCollectBlip();
	}
	
	void TryCollectBlip()
	{
//		Debug.Log(blippos+","+playerpos);
		if (blippos.Equals(playerpos))
		{
			source.PlayOneShot(sndcollect);
			playerlength++;
			SetBlipPos();
		}
	}
	
	public TextMesh tm;
	private float time=0;
	// Update is called once per frame
	void Update () {
		if (Input.GetKeyDown(KeyCode.R))
		{
			Application.LoadLevel("MainScene");
		}
		if (dead)
		{
			return;
		}
		if (awake==false)
		{
			if (Input.GetKeyDown(KeyCode.Space))
			{
				tm.gameObject.active=false;
				awake=true;
			}
			return;
		}
		if (Input.GetKeyDown(KeyCode.LeftArrow))
		{
			SetPlayerPos(false);
			playerup = playerup.RotateAround(playerforward,-1);
			TryCollectBlip();
			SetPlayerPos();
		}
		if (Input.GetKeyDown(KeyCode.RightArrow))
		{
			SetPlayerPos(false);
			playerup = playerup.RotateAround(playerforward,1);
			TryCollectBlip();
			SetPlayerPos();
		}
		
		if (Input.GetKeyDown(KeyCode.UpArrow))
		{
			SetPlayerPos(false);
			Triple.Direction right = playerup.Cross(playerforward);
			playerforward = playerforward.RotateAround(right,1);
			playerup = playerup.RotateAround(right,1);
			TryCollectBlip();
			SetPlayerPos();
		}
		if (Input.GetKeyDown(KeyCode.DownArrow))
		{
			SetPlayerPos(false);
			Triple.Direction right = playerup.Cross(playerforward);
			playerforward = playerforward.RotateAround(right,-1);
			playerup = playerup.RotateAround(right,-1);
			TryCollectBlip();
			SetPlayerPos();
		}
		time+=Time.deltaTime;
		if (time>framelength)
		{
			time-=framelength;
			GameTick();
		}
	}
	
	private void GameTick()
	{
		//SetPlayerPos(false);
		playerpos = playerpos + playerforward.ToTriple();
		TryCollectBlip();
		SetPlayerPos();
		UpdateTail();
		source.PlayOneShot(sndtick);
	}	
	
	private void UpdateTail()
	{
		
		if ((beamposes.Count==0) || (playerpos.Equals(beamposes[beamposes.Count-1])==false))
		{
			beamposes.Add(playerpos);
			beamdirs.Add(playerforward);
			beamdirs2.Add(playerup);
		}
		while (beamposes.Count>playerlength)
		{
			beamposes.RemoveAt(0);
			beamdirs.RemoveAt(0);
			beamdirs2.RemoveAt(0);
		}
		while (beamobs.Count<beamposes.Count-1)
		{			
			var go = (GameObject)Instantiate(beamprefab);
			go.transform.parent=beambox.transform;
			beamobs.Add(go);
		}
		
		for (int i=0;i<beamposes.Count-1;i++)
		{
			Triple beampos = beamposes[i];
			Triple.Direction beamforward = beamdirs[i];
			Triple.Direction beamup = beamdirs2[i];
			GameObject bo = beamobs[i];
			Vector3 targetposition = PointToCoord(beampos);
			Quaternion targetrotation = Quaternion.LookRotation(beamforward.ToVector(),beamup.ToVector());
			bo.GetComponent<TweenTo>().Tween(targetposition,targetrotation);
		}
		CheckDeath();
	}
	
	private void CheckDeath()
	{
		for (int i=0;i<beamobs.Count;i++)
		{			
			Triple beampos = beamposes[i];
			Triple.Direction beamforward = beamdirs[i];
			Triple.Direction beamup = beamdirs2[i];
			Triple.Direction beamright = beamup.Cross(beamforward);
			Triple beampos2d = beampos - beampos.ProjectOnto(beamright);
			Triple playerpos2d = playerpos - playerpos.ProjectOnto(beamright);
			if (beampos2d.Equals(playerpos2d))
			{
				dead=true;
			}
		}	
		if (playerpos.x<0 || playerpos.y<0 || playerpos.z<0 || playerpos.x>=width || playerpos.y>=width || playerpos.z>=width)
		{
			dead=true;
		}
		if (dead)
		{
			int score = (playerlength-inittraillength);
			int highscore = PlayerPrefs.GetInt("highscore",0);
			if (score>highscore)
			{
				PlayerPrefs.SetInt("highscore",score);
				highscore=score;
			}
			tm.text="YOU DIED\nscore: " + score+"\nhighscore: " + highscore+"\nPress R to restrt";
			tm.gameObject.active=true;
			source.PlayOneShot(snddead);
			//Application.LoadLevel("MainScene");	
		}
		
	}
	
	private void SetPlayerPos(bool tween=true)
	{
		Vector3 targetpos = PointToCoord(playerpos);
		Quaternion targetrot = Quaternion.LookRotation(playerforward.ToVector(),playerup.ToVector());
		if (tween)
		{
			GetComponent<TweenTo>().Tween(targetpos,targetrot);
		}
		else 
		{
	//		transform.position=targetpos;
	//		transform.rotation=targetrot;
		}
		
		CheckDeath();
	}
}

