using UnityEngine;
using System;
using System.Collections.Generic;
using System.Runtime.Serialization;

[Serializable]
public struct Triple  
{
	[Serializable]
	public enum Direction
	{
		Up = 0,
		Down = 1,
		North = 2,
		South = 3,
		East = 4,
		West = 5,
		MAX = 6
	}
	
	public readonly int x;
	public readonly int y;
	public readonly int z;
	
	public static readonly Triple FarAway = new Triple(1000,1000,1000);
	
	public Triple Rotate (int rx, int ry, int rz, bool flip)
	{
		int nx = x;
		int ny = y;
		int nz = z;
		//rotate about x
		for (int i = 0; i < rx; i++)
		{
			int temp = ny;
			ny = -nz;
			nz = temp;
		
		}
		//rotate about y
		for (int j = 0; j < rx; j++)
		{
			int temp = nz;
			nz = -nx;
			nx = temp;		
		}
		//rotate about z
		for (int k = 0; k < rx; k++)
		{
			
			int temp = nx;
			nx = -ny;
			ny = temp;
		}
		
		
		if (flip)
			nx = -nx;
		
		return new Triple(nx, ny, nz);
	}
	
	
	public Vector3 ToVector ()
	{
		return new Vector3(x, y, z);
	}				
	
	public static Triple zero = new Triple(0,0,0);	
	
	public Triple (int _x, int _y, int _z)
	{
		x = _x;
		y = _y;
		z = _z;		
	}
	
	public object Clone ()
	{
		return this;
	}
	
	//looks north south east and west, if not on one of these lines, returns -1
	//+ out returns max 
	public static int HandleDistance (Triple a, Triple b, out Direction directionfromAtoB)
	{
		directionfromAtoB = Direction.MAX;
		int n = 0;
		int res = 0;
		if (a.x == b.x)
		{
			n++;
		}
		else
		{
			res = Mathf.Abs(a.x - b.x);
			if (a.x < b.x)
			{
				directionfromAtoB = Direction.East;
			}
			else
			{
				directionfromAtoB = Direction.West;
			}
		}
		
		
		if (a.y == b.y)
		{
			n++;
		}
		else
		{
			res = Mathf.Abs(a.y - b.y);
			if (a.y < b.y)
			{
				directionfromAtoB = Direction.Up;
			}

			else
			{
				directionfromAtoB = Direction.Down;
			}
		}
		
		
		if (a.z == b.z)
		{
			n++;
		}

		else
		{
			res = Mathf.Abs(a.z - b.z);
			if (a.z < b.z)
			{
				directionfromAtoB = Direction.North;
			}

			else
			{
				directionfromAtoB = Direction.South;
			}
		}
		
		if (n == 2)
		{
			return res;
		}
		else
		{
			directionfromAtoB = Direction.MAX;
			return -1;
		}
	}
	
	public static int ManhattanDistance (Triple a, Triple b)
	{
		return (b - a).ManhattanSize();
	}
	
	public int ManhattanSize ()
	{
		return Mathf.Abs(x) + Mathf.Abs(y) + Mathf.Abs(z);
	}
	
	public int Trace ()
	{
		return x+y+z;
	}
	
	public Triple ProjectOnto (Triple.Direction dir)
	{
		Triple dirt = dir.ToTriple();
		
		return new Triple(x*dirt.x,y*dirt.y,z*dirt.z);
	}
		
	public Triple OrthoComponentTo (Triple.Direction dir)
	{
		return this - this.ProjectOnto(dir);
	}
	

	public bool Equals (Triple other)
	{
		return x == other.x && y == other.y && z == other.z;
	}



	public override string ToString ()
	{
		return "(" + x.ToString() + "," + y.ToString() + "," + z.ToString() + ")";		
	}
	
	public List<Triple> Neighbours (bool includethis)
	{
		List<Triple> result = new List<Triple>();
		
		if (includethis)
		{
			result.Add(this);
		}
		
		result.Add(new Triple(x + 1, y, z));
		result.Add(new Triple(x - 1, y, z));
		result.Add(new Triple(x, y + 1, z));
		result.Add(new Triple(x, y - 1, z));
		result.Add(new Triple(x, y, z + 1));
		result.Add(new Triple(x, y, z - 1));
		return result;
	}
	
	public Triple Move (Direction d)
	{
		switch (d)
		{
			case Direction.Up:
				return new Triple(x, y + 1, z);
			case Direction.Down:
				return new Triple(x, y - 1, z);
			case Direction.North:
				return new Triple(x, y, z + 1);
			case Direction.South:
				return new Triple(x, y, z - 1);
			case Direction.East:
				return new Triple(x + 1, y, z);
			case Direction.West:
				return new Triple(x - 1, y, z);
		}
		return default(Triple);
	}
	
	public static Triple Cross (Triple t1, Triple t2)
	{
		return new Triple(t1.y * t2.z - t1.z * t2.y, t1.z * t2.x - t2.z * t1.x, t1.x * t2.y - t2.x * t1.y);
	}	
	
	public static Triple operator + (Triple t1, Triple t2)
	{
		return new Triple(t1.x + t2.x, t1.y + t2.y, t1.z + t2.z);
	}

	public static Triple operator - (Triple t1, Triple t2)
	{
		return new Triple(t1.x - t2.x, t1.y - t2.y, t1.z - t2.z);
	}
	
	public static Triple operator * (int s, Triple t)
	{
		return new Triple(s*t.x,s*t.y,s*t.z);
	}
	
	public static Triple operator * (Triple t, int s)
	{
		return new Triple(s * t.x, s * t.y, s * t.z);
	}
	
	public static Triple operator - (Triple t)
	{
		return new Triple(-t.x, -t.y, -t.z);
	}
}
